/*
 * CPU-specific routines from U-Boot.
 * See: uboot-xburst/files/arch/mips/cpu/xburst/cpu.c
 * See: u-boot/arch/mips/include/asm/cacheops.h
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "xburst_types.h"
#include "sdram.h"

void flush_icache_all(void)
{
	u32 addr, t = 0;

	asm volatile ("mtc0 $0, $28"); /* Clear Taglo */
	asm volatile ("mtc0 $0, $29"); /* Clear TagHi */

	for (addr = KSEG0; addr < KSEG0 + CONFIG_SYS_ICACHE_SIZE;
	     addr += CONFIG_SYS_CACHELINE_SIZE) {
		asm volatile (
			".set mips3\n\t"
			" cache %0, 0(%1)\n\t"
			".set mips2\n\t"
			:
			: "I" (Index_Store_Tag_I), "r"(addr));
	}

	/* invalicate btb */
	asm volatile (
		".set mips32\n\t"
		"mfc0 %0, $16, 7\n\t"
		"nop\n\t"
		"ori %0,2\n\t"
		"mtc0 %0, $16, 7\n\t"
		".set mips2\n\t"
		:
		: "r" (t));
}

void flush_dcache_all(void)
{
	u32 addr;

	for (addr = KSEG0; addr < KSEG0 + CONFIG_SYS_DCACHE_SIZE; 
	     addr += CONFIG_SYS_CACHELINE_SIZE) {
		asm volatile (
			".set mips3\n\t"
			" cache %0, 0(%1)\n\t"
			".set mips2\n\t"
			:
			: "I" (Index_Writeback_Inv_D), "r"(addr));
	}

	asm volatile ("sync");
}

void flush_cache_all(void)
{
	flush_dcache_all();
	flush_icache_all();
}

void handle_error_level(void)
{
	asm volatile(
		"mfc0 $t3, $12\n" /* CP0_STATUS */
		"nop\n"
		"li $t4, 0xfffffffb\n" /* ERL = 0 */
		"and $t3, $t3, $t4\n"
		"mtc0 $t3, $12\n"
		"nop\n");
}

void enable_interrupts(void)
{
	asm volatile(
		"mfc0 $t3, $12\n" /* CP0_STATUS */
		"nop\n"
		"li $t4, 0x0000fc01\n" /* IE = enable interrupts */
		"or $t3, $t3, $t4\n"
		"mtc0 $t3, $12\n"
		"nop\n");
}

void init_interrupts(void)
{
	/* Set exception registers. */

	asm volatile(
		"mtc0 $zero, $18\n"	/* CP0_WATCHLO */
		"nop\n"
		"li $t3, 0x00800000\n"	/* IV = 1 (use 0x80000200 for interrupts) */
		"mtc0 $t3, $13\n"	/* CP0_CAUSE */
		"nop\n"
		"mtc0 $zero, $12\n"	/* CP0_STATUS */
		"nop\n"
		"mtc0 $zero, $15\n"	/* CP0_EBASE (should be zero anyway) */
		"nop\n");
}

void init_tlb(void)
{
	asm volatile(
		"li $t0, 0x001fe000\n"	/* 1MB */
		"mtc0 $t0, $5\n"	/* CP0_PAGEMASK */
		"nop\n"
		"mtc0 $zero, $6\n"	/* CP0_WIRED */
		"nop\n"

		/* Set physical address. */

		"li $t0, 0x00000003\n"	/* 0x000000.. global, valid */
		"mtc0 $t0, $2\n"	/* CP0_ENTRYLO0 */
		"nop\n"
		"li $t0, 0x00000003\n"	/* 0x000000.. global, valid */
		"mtc0 $zero, $3\n"	/* CP0_ENTRYLO1 */
		"nop\n"

		/* Set virtual address. */

		"li $t0, 0x80000000\n"	/* 0x80000... */
		"mtc0 $t0, $10\n"	/* CP0_ENTRYHI */
		"nop\n"

		/* Set index. */

		"mtc0 $zero, $0\n"	/* CP0_INDEX */
		"nop\n"
		"tlbwi\n");
}
